package jp.org.tomotaro.aop;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.After;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;

@Component
@Aspect
public class SampleIntercepter {
	@Pointcut("execution(* jp.org.tomotaro.aop.*.*(..))")
	public void pointCut()	{
	}

	//@Before("execution(* jp.org.tomotaro.aop.*.*(..))")
	@Before("pointCut()")
	public void before(JoinPoint joinPoint) throws Throwable {
		System.out.println("Before Method started excuting...");
		Signature signature = joinPoint.getSignature();
		System.out.println(signature.getDeclaringTypeName()+ "." + signature.getName());
	}
	
	@After("pointCut()")
	public void after(JoinPoint joinPoint) throws Throwable {
		System.out.println("After Method executed...");
		Signature signature = joinPoint.getSignature();
		System.out.println(signature.getDeclaringTypeName() + "." + signature.getName());		
	}
	
	@AfterReturning(value = "pointCut()",  returning = "returnValue")
	public void afterReturning(JoinPoint joinPoint, Object returnValue) throws Throwable {
		System.out.println("After Method returning...");
		Signature signature = joinPoint.getSignature();
		System.out.println(signature.getDeclaringTypeName() + "." + signature.getName());
		System.out.println("returnValue=" + returnValue.toString());
	}
	
	@AfterThrowing(value = "pointCut()", throwing = "throwable")
	public void afterThrowing(JoinPoint joinPoint, Throwable throwable) {
		System.out.println("After Exception throwing...");
		Signature signature = joinPoint.getSignature();
		System.out.println(signature.getDeclaringTypeName() + "." + signature.getName());
		System.out.println(throwable.getMessage());
	}
	
	@Around(value = "pointCut()")
	public Object around(ProceedingJoinPoint joinPoint) throws Throwable {
		System.out.println("Around start...");
		Signature signature = joinPoint.getSignature();
		System.out.println(signature.getDeclaringTypeName() + "." + signature.getName());

		Object returnValue = joinPoint.proceed();
		
		System.out.println("Around end...");
		if(returnValue != null) {
			System.out.println("Around returnValue=" + returnValue.toString());
		}
		
		return returnValue;
	}
}
